package DomainHelper;
use strict;
use warnings;
use File::Basename;

sub new {
    my $class = shift;
    my $self  = {};
    bless( $self, $class );
    return $self;
}

sub readConfig {
    my $self = shift;
    my %map;
    my $dir = File::Basename::dirname($0);
    open( RD, "<$dir/config.ini" );
    while ( my $line = <RD> ) {
        next if($line =~ /^#/);
        if ( $line =~ /ThreaDom_path/ ) {
            $line =~ /"(\S+)"/;
            $map{'THREADOM_PATH'} = $1;
        }
        elsif ( $line =~ /Data_dir_base/ ) {

            $line =~ /"(\S+)"/;
            $map{'DIR_BASE'} = $1;

        }
        elsif ( $line =~ /Dataset_name/ ) {

            $line =~ /"(\S+)"/;
            $map{'DATASET_NAME'} = $1;
        }
        elsif ( $line =~ /I-TASSER_path/ ) {

            $line =~ /"(\S+)"/;
            $map{'I-TASSER_PATH'} = $1;

        }
        elsif ( $line =~ /DSSP_path/ ) {

            $line =~ /"(\S+)"/;
            $map{'DSSP_PATH'} = $1;

        }
        elsif ( $line =~ /DomainParser_path/ ) {

            $line =~ /"(\S+)"/;
            $map{'DOMAINPARSER_PATH'} = $1;

        }
        elsif ( $line =~ /pdb_all_path/ ) {

            $line =~ /"(\S+)"/;
            $map{'PDBALL_PATH'} = $1;

        }elsif($line =~ /Data_path/){

            $line =~ /"(\S+)"/;
            $map{'DATA_PATH'} = $1;
      }elsif($line =~ /cutoffs/){
            $line =~ /"(.*)"/;
            $map{'CUTOFFS'} = $1;

     }

    }


close(RD);
while ( my ( $key, $value ) = each %map ) {

  if ( not $value ) {
      print("No valid value of $key\n .Check the file :config.ini\n");
      exit;
  }
}

return %map;

}

sub readDatasetList {
    my $self = shift;
    my ( $fileName, $num ) = @_;
    my ( %list, %listmap, @arrayList );
    my ( $numMD, $numSD, $numBoundary, $num_list );
    my $helper = new DomainHelper();
    $numMD       = 0;
    $numSD       = 0;
    $num_list    = 0;
    $numBoundary = 0;
    open RD, '<', $fileName
      or die "$0 : failed to open  input file '$fileName' : $!\n";

    while ( my $line = <RD> ) {
        $line =~ /(\S+)\t(\S+)\t(\S)\t(\S+)/;
        my $chainId   = $1;
        my $length    = $2;
        my $numDomain = $3;
        my $domains   = $4;
        if ( $numDomain >= $num ) {

            push( @arrayList, $chainId );
            my %map;
            $map{$chainId}     = $chainId;
            $map{Length}       = $length;
            $map{DomainNumber} = $numDomain;
            $map{Domains}      = $domains;
            $num_list++;
            $listmap{$chainId} = \%map;
            if ( $numDomain >= 2 ) {
                $numMD++;
            }
            else {
                $numSD++;
            }
            my @array = $helper->parseBoundary($domains);
            foreach my $element (@array) {
                if ( $element > 40 and $element < $length - 40 ) {
                    $numBoundary++;
                }
            }

        }
    }

    $list{List}           = \@arrayList;
    $list{TotalNumber}    = $num_list;
    $list{SDNumber}       = $numSD;
    $list{MDNumber}       = $numMD;
    $list{BoundaryNumber} = $numBoundary;

    $list{MinDomainNum} = $num;
    $list{Content}      = \%listmap;
    close RD
      or warn "$0 : failed to close input file '$fileName' : $!\n";
    return \%list;
}    # ----------  end of subroutine readlist  ----------

sub readMyCathDB {

    my ( $self, $dbFile ) = @_;
    my $line = "";
    my %myCathDB;
    open( RD, "<$dbFile" );

    while ( $line = <RD> ) {
        chop($line);
        my $chainId = substr( $line, 0, 5 );
        $myCathDB{$chainId} = $line;
    }
    close(RD);
    return %myCathDB;
}    # ----------  end of subroutine initalizeDB  ----------

sub searchArrayIndex {
    my ( $self, $arrayRef, $value ) = @_;
    my $nIndex_N = 0;
    my $nIndex_C = 0;
    my $nIndex   = 0;
    for ( my $i = 0 ; $i <= $#$arrayRef ; $i++ ) {
        if ( $$arrayRef[$i] <= $value ) {
            $nIndex_N = $i;
        }
        else {
            last;
        }
    }
    if ( $$arrayRef[$nIndex_N] != $value ) {
        for ( my $i = $#$arrayRef ; $i >= 0 ; $i-- ) {
            if ( $$arrayRef[$i] >= $value ) {
                $nIndex_C = $i;
            }
            else {
                last;
            }
        }
        $nIndex =
          abs( $value - $$arrayRef[$nIndex_N] ) <
          abs( $value - $$arrayRef[$nIndex_C] ) ? $nIndex_N : $nIndex_C;
    }
    else {
        $nIndex = $nIndex_N;
    }
    return $nIndex;
}    # ----------  end of subroutine searchArrayIndex  ----------

sub parseCathRecord {
    my ( $self, $record ) = @_;
    my %data;

    $record =~ /(\S+)\s(\S+)\s(\S+)\s+(.*)/;

    my $chainName    = $1;    #chainName
    my $numDomains   = $2;    #number of domain "D02"
    my $numFragments = $3;    #number of Fragment "F01"
    my $detail       = $4;    #other information

    $numDomains =~ /\S([1-9]+)/;    # "D01" ->1
    $numDomains = $1;
    $numFragments =~ /\S([1-9]+)/;    #  "F01"->1
    $numFragments         = $1;
    $data{"chainName"}    = $chainName;
    $data{"numDomains"}   = $numDomains;
    $data{"numFragments"} = $numFragments;

    for ( my $i = 1 ; $i <= $numDomains ; $i++ ) {
        my $scale = "";

        $detail =~ /\s*(\d+)\s(.*)/;
        my $num = $1;
        $detail = $2;

        for ( my $j = 0 ; $j < $num ; $j++ ) {

            $detail =~ /\s*\S\s+(\S+)\s\S\s\S\s+(\S+)\s\S(.*)/;
            if ( $scale eq "" ) {
                $scale = "$1-$2";
            }
            else {
                $scale .= "|$1-$2";
            }
            $detail = $3;
        }
        $data{"domain$i"} = $scale;

    }

    return %data;
}    # ----------  end of subroutine parseCathRecord  ----------

sub readDpDB {
    my ( $self, $inFile ) = @_;
    my ( $rd, %dpDB );
    my $line = "";
    if ( -e "$inFile" ) {
        open( $rd, "<$inFile" );
        while ( $line = <$rd> ) {
            $line =~ /(\S+)\t(.*)/;
            my $template = $1;

            my $links = $2;
            my @array = split( /\t/, $links );
            $dpDB{$template} = \@array;
        }

        close($rd);
    }
    else {
        print "$inFile not exist.";
    }
    return %dpDB;
}

sub writeDpDB {
    my ( $self, $ref_dpDB, $outFile ) = @_;
    my $wr;
    open( $wr, ">$outFile" );
    while ( my ( $key, $value ) = each %$ref_dpDB ) {
        printf $wr "$key\t@$value\n";
    }
    close($wr);
}

sub normalizeTemplateName {
    my ( $self, $templateName ) = @_;
    my $value = "";
    $templateName =~ s/_//g;
    $templateName =~ tr/[A-Z]/[a-z]/;
    my $str1 = substr( $templateName, 0, 4 );
    my $str2 = substr( $templateName, 4, 1 );

    #The below rule has some drawback
    if ( $str2 eq "" ) {
        $str2 = "A";
    }
    elsif ( $str2 eq "1" ) {
        $str2 = "A";
    }
    elsif ( $str2 eq "2" ) {
        $str2 = "A";
    }
    elsif ( $str2 eq "3" ) {
        $str2 = "A";
    }
    $value = $str1 . uc($str2);
    return $value;
}    # ----------  end of subroutine normalizeTemplateName  ----------

sub configDomainParser {
    my ( $self, $dsspPath ) = @_;
    $ENV{'DSSP_PATH'} = $dsspPath;
}

sub callDomainParser {
    my ( $self, $chainId ) = @_;
    my $str = "";
    my @boundaries;
    my $index   = 0;
    my $pdbFile = "/home/yzhang/PDBall/$chainId.pdb";
    if ( !-e "$pdbFile" ) {
        print "$pdbFile not exist\n";
        return @boundaries;
    }
    `/bin/cp "$pdbFile" ./tmp.pdb`;
    my $domain = `/home/zhidongx/bin/domainparser2 tmp`;
    `rm tmp.pdb`;
    if ( $domain ne "" ) {
        $domain =~ /\S+\s+(\S+)\s+(\d+)/;
        my $Lch      = $1;
        my $n_domain = $2;

        #        printf "Lch=$Lch   n_domain=$n_domain\n";
        if ( $n_domain == 1 ) {
            $boundaries[0] = "1";
            $boundaries[1] = "$Lch";
        }
        else {
            if ( $domain =~ /\S+\s+\S+\s+(\d+)\s+(.+)/ ) {
                my $DOM      = $2;
                my @doms     = split ' ', $DOM;
                my $i_domain = 0;
                foreach my $dom (@doms) {
                    $i_domain++;
                    my @ijs = split ';', $dom;
                    foreach my $ij (@ijs) {
                        $ij =~ /(\d+)-(\d+)/;
                        $boundaries[$index] = $1;
                        $index++;
                        $boundaries[$index] = $2;
                        $index++;
                    }
                }
            }
        }
        @boundaries = sort { $a <=> $b } @boundaries;

        #        print "$chainId:Domainparser:@boundaries\n";
    }
    else {
        print "DomainPaser Failed!";
    }
    return @boundaries;
}

sub parseDomainRecord {
    my ( $self, $record ) = @_;
    my %data;
    if ( not $record ) {
        print "parameter is null! check it please.";
        return %data;
    }
    else {
        $record =~ /(\S+)\t(\S+)\t(\S+)\t(\S+)/;
        my $chainId    = $1;
        my $numDomains = $3;
        my $details    = $4;
        $data{"chainName"}  = $chainId;
        $data{"numDomains"} = $numDomains;
        my @array = $self->parseDomain($details);
        for ( my $i = 1 ; $i <= $numDomains ; $i++ ) {
            $data{"domain$i"} = $array[ $i - 1 ];
        }
    }

    return %data;
}    # ----------  end of subroutine getCathDomainInfo  ----------

sub parseDomain {
    my ( $self, $details ) = @_;
    chop($details);    #$detail="(1-100)(110-200)"
    $details =~ s/\(//g;
    my @array = split( '\)', $details );
    return @array;     #return array[0]="1-100",array[1]="110-200"
}

sub domain2boundary {
    my $self      = shift;
    my ($domains) = @_;
    my @result    = $self->parseBoundary($domains);
    return @result;
}

sub parseBoundary {
    my $self = shift;
    my ($domains) = @_;
    my @tempStart;
    my @tempEnd;
    my $num = 0;
    my @result;
    while ( $domains ne "" ) {
        if ( $domains =~ m/,/ ) {
            $domains = "(1-2)";
        }
        $domains =~ /\((\d+)-(\d+)\)(.*)/;
        $tempStart[$num] = $1;
        $tempEnd[$num]   = $2;
        $domains         = $3;
        $num++;
    }
    for ( my $i = 0 ; $i < $num ; $i++ ) {
        $result[ 2 * $i ] = $tempStart[$i];
        $result[ 2 * $i + 1 ] = $tempEnd[$i];
    }

    return @result;
}

sub parseLinks {
    my $self = shift;
    my ( $links, $refinement ) = @_;
    my @tempStart;
    my @tempEnd;
    my $num = 0;
    my @result;
    my $links2 = $links;
    while ( $links2 ne "" ) {

        $links2 =~ /\((\d+)-(\d+)\)(.*)/;
        $tempStart[$num] = $1;
        $tempEnd[$num]   = $2;
        $links2          = $3;
        $num++;
        last if ( not defined $links2 );
    }
    if ( $num > 0 ) {
        for ( my $i = 0 ; $i < $num ; $i++ ) {
            $result[ 3 * $i ] = $tempStart[$i];
            $result[ 3 * $i + 1 ] =
              int( ( $tempEnd[$i] + $tempStart[$i] ) / 2 );
            $result[ 3 * $i + 2 ] = $tempEnd[$i];
        }

        #			for ( my $i = 1 ; $i < $#result ; $i++ ) {
        #
        #				if ( abs( $result[$i] - $result[ $i - 1 ] ) < 80 )
        #				{    #print " init:@result\n";
        #					$result[$i] =
        #					  int( ( $result[$i] + $result[ $i - 1 ] ) / 2 );
        #					splice( @result, $i - 1, 1 );
        #					$i--;
        #				}
        #
        #			}

        if ( $refinement > 0 ) {
            $self->refineBoundary( \@result );
        }
    }
    else {
        $result[0] = -1;
    }

    return @result;
}

sub domains2Linkers {
    my $self = shift;
    my ( $domains, $length );
    my %map;

    my $helper = new DomainHelper();
    my @array  = $helper->parseBoundary($domains);

    my $linkNum = ( $#array + 1 ) / 2 - 1;
    my $header  = "";
    my $tail    = "";

    if ( $array[0] > 1 ) {
        $header = "1-$array[0]";
    }

    if ( $array[$#array] < length ) {
        $tail = "$array[$#array]-1";
    }
    $map{LinksNumber} = $linkNum;
    $map{Header}      = $header;
    $map{Tail}        = $tail;
    for ( my $i = 1 ; $i <= $linkNum ; $i += 2 ) {
        $map{"Linker$i"} = "$array[2*$i-1]-$array[2*$i]";
    }
    return %map;
}

sub links2Domains {
    my $self = shift;
    my ( $links, $length ) = @_;

    my @array =
      $self->parseLinks( $links, 1 );    #return start-pos-end-start-pos-end..

    if ( $#array == 0 ) {
        return "(1-$length)";
    }
    my $numLinks = ( $#array + 1 ) / 3;
    my $domains  = "";

    my @arrayBoundary;
    for ( my $i = 0 ; $i < $numLinks ; $i++ ) {
        push( @arrayBoundary, $array[ 3 * $i + 1 ] );
        push( @arrayBoundary, $array[ 3 * $i + 1 ] + 1 );
    }
    push( @arrayBoundary, 1 );
    push( @arrayBoundary, $length );
    @arrayBoundary = sort { $a <=> $b } (@arrayBoundary);
    for ( my $i = 0 ; $i < $numLinks + 1 ; $i++ ) {
        $domains .= "($arrayBoundary[2*$i]-$arrayBoundary[2*$i+1])";
    }
    return $domains;

}

sub refineBoundary {
    my $self               = shift;
    my ($boundaryArrayRef) = @_;
    my $len                = ( $#$boundaryArrayRef + 1 ) / 3;
    my $num                = $len;
    my @array;
    if ( $len > 1 ) {
        for ( my $i = 1 ; $i < $len ; $i++ ) {
            if (
                abs(
                    $$boundaryArrayRef[ 3 * $i + 1 ] -
                      $$boundaryArrayRef[ 3 * ( $i - 1 ) + 1 ]
                ) < 20
              )
            {
                $$boundaryArrayRef[ 3 * ( $i - 1 ) + 1 ] = int(
                    (
                        $$boundaryArrayRef[ 3 * $i + 1 ] +
                          $$boundaryArrayRef[ 3 * ( $i - 1 ) + 1 ]
                    ) / 2
                );
                splice( @$boundaryArrayRef, 3 * ( $i - 1 ) + 2, 3 );
                $len--;

            }
        }
        if ( $num > $len ) {
            $self->refineBoundary($boundaryArrayRef);
        }

    }
    return $boundaryArrayRef;
}

1;
